from .Contours import Circle, Annulus, AnnulusSector, Rectangle
from .RootFinder import findRoots
from .DemoRootFinder import demo_findRoots
from .CountRoots import count_enclosed_roots
from .CxDerivative import CxDeriv


"""
References
----------
[DSZ] "Locating all the Zeros of an Analytic Function in one Complex Variable"
	M.Dellnitz, O.Schutze, Q.Zheng, J. Compu. and App. Math. (2002), Vol.138, Issue 2
[BVP] Gakhov, F. D. "Boundary value problems", section 12 (2014), Elsevier.
[DL] "A Numerical Method for Locating the Zeros of an Analytic function", 
	L.M.Delves, J.N.Lyness, Mathematics of Computation (1967), Vol.21, Issue 100
[KB] "Computing the Zeros of Anayltic Functions", Peter Kravanja, Marc Van Barel,
	Springer (2000)
"""

from __future__ import division
import random
import warnings
import numpy as np
import scipy.integrate
import scipy.linalg
from scipy import pi, exp, sin, log
import scipy

from .CountRoots import count_enclosed_roots, prod
from .RootFinder import findRoots
from .DemoRootFinder import demo_findRoots
from .Paths import ComplexLine, ComplexArc
from .Misc import doc_tab_to_space, docstrings

class Contour(object):
	def __init__(self, segments):
		self.segments = np.array(segments, dtype=object)

	def __call__(self, t):
		t = np.array(t)
		N = len(self.segments)
		segmentIndex = np.array(N*t, dtype=int)
		segmentIndex = np.mod(segmentIndex, N)
		segment = self.segments[segmentIndex]

		if hasattr(segmentIndex, '__iter__'):
			return np.array([self.segments[i](N*t[ti]%1) for ti, i in enumerate(segmentIndex)])
		else:
			return self.segments[segmentIndex](N*t%1)

	def plot(self, *args, **kwargs):
		for segment in self.segments:
			segment.plot(*args, **kwargs)

	def sizePlot(self):
		import matplotlib.pyplot as plt
		t = np.linspace(0,1,1000)
		z = self(t)
		xpad = (max(np.real(z))-min(np.real(z)))*0.1
		ypad = (max(np.imag(z))-min(np.imag(z)))*0.1

		xmin = min(np.real(z))-xpad
		xmax = max(np.real(z))+xpad
		ymin = min(np.imag(z))-ypad
		ymax = max(np.imag(z))+ypad
		plt.xlim([xmin, xmax])
		plt.ylim([ymin, ymax])

	def show(self, saveFile=None, *args, **kwargs):
		""" 
		Shows the contour as a 2D plot in the complex plane.

		Parameters
		==========
		saveFile : str (optional)
			If given then the plot will be saved to disk with name 'saveFile' instead of being shown.
		"""
		import matplotlib.pyplot as plt
		self.sizePlot()
		self.plot(*args, **kwargs)

		if saveFile is not None:
			plt.savefig(saveFile)
			plt.close()
		else:
			plt.show()

	def subdivisions(self, axis='alternating'):
		""" 
		A generator of possible subdivisions of the contour, starting with an equal subdivision. 
		
		Parameters
		----------
		axis : str, 'alternating' or any element of self.axisName which is defined in a Contour subclass
			The axis along which the line subdividing the contour is a constant (eg. subdividing a circle
			along the radial axis will give an outer annulus and an inner circle).  If alternating then
			the dividing axis will always be different to the dividing axis used to create the contour
			which is now being divided.

		Returns
		-------
		Generator of possible subdivisions
		"""
		if axis == 'alternating':
			# if the box to be subdivided was itself created by subdivision then alternate the axis of subdivision
			if hasattr(self,'_createdBySubdivisionAxis'):
				axis = (self._createdBySubdivisionAxis + 1)%len(self.axisName)
			else:
				axis = 0

		for divisionFactor in divisionFactorGen():
			yield self.subdivide(axis, divisionFactor)

	def integrate(self, f, absTol=0, relTol=1e-12, rombergDivMax=10, method='quad', show=False):
		""" Integrate around the contour, same arguments the integrate method for ComplexPath """
		return sum([segment.integrate(f, absTol, relTol, rombergDivMax, method, show) for segment in self.segments])

	def count_roots(self, *args, **kwargs):
		return count_enclosed_roots(self, *args, **kwargs)

	def approximate_roots(self, f, df=None, absTol=1e-12, relTol=1e-12, integerTol=0.25, divMax=10, rootTol=1e-8):
		N = self.count_roots(f, df, integerTol, divMax)

		if N == 0:
			return (), ()

		mu = prod(self, f, df, lambda z: z, lambda z: 1, absTol, relTol, divMax)[0]/N
		phiZeros = [[],[mu]]

		def phiFunc(i):
			if len(phiZeros[i]) == 0:
				return lambda z: 1
			else:
				coeff = np.poly(phiZeros[i])
				return lambda z: np.polyval(coeff, z)
			
		# print('mu', mu)

		err_stop = 1e-8

		# initialize G_{pq} = <phi_p, phi_q>
		G = np.zeros((N,N), dtype=np.complex128)
		G[0,0] = N # = <phi0, phi0>

		# initialize G1_{pq} = <phi_p, phi_1 phi_q>
		G1 = np.zeros((N,N), dtype=np.complex128)
		ip, err = prod(self, f, df, phiFunc(0), lambda z: phiFunc(1)(z)*phiFunc(0)(z), absTol, relTol, divMax)
		G1[0,0] = ip

		take_regular = True

		r, t = 1, 0
		while r+t<N:
			# define the next FOP of degree r+t+1
			k = r+t+1

			# Add new values to G
			p = r+t
			G[p, 0:p+1] = [prod(self, f, df, phiFunc(p), phiFunc(q), absTol, relTol, divMax)[0] for q in range(r+t+1)]
			G[0:p+1, p] = G[p, 0:p+1] # G is symmetric

			# Add new values to G1
			phi1 = phiFunc(1)
			G1[p, 0:p+1] = [prod(self, f, df, phiFunc(p), lambda z: phi1(z)*phiFunc(q)(z), absTol, relTol, divMax)[0] for q in range(r+t+1)]
			G1[0:p+1, p] = G1[p, 0:p+1] # G1 is symmetric

			# print('G', G[:p+1,:p+1])
			# print('G1', G1[:p+1,:p+1])

			# The regular FOP only exists if H is non-singular.
			# An alternate citeration given by [KB] is to proceed as if it is regular and
			# then compute its zeros.  If any are arbitary or infinite then this
			# polynomial should instead be defined as an inner polynomial.
			# Here, an inner polynomial instead if any of the computed
			# roots are outside of the interior of the contour.
			polyRoots = scipy.linalg.eig(G1[:p+1,:p+1], G[:p+1,:p+1])[0]+mu
			# print('polyRoots', polyRoots)
			if np.all([self.contains(z) for z in polyRoots]):
				# define a regular polynomial
				phiZeros.append(polyRoots)
				r, t = r+t+1, 0
				# print('regular', r+t)

				# if any of these elements are not small then continue
				allSmall = True
				phiFuncLast = phiFunc(-1)
				for j in range(N-r):
					ip, err = prod(self, f, df, lambda z: phiFuncLast(z)*(z-mu)**j, phiFuncLast, absTol, relTol, divMax)

					# if not small then carry on
					# print(j, 'of', N-r, 'stop?', ip)
					### XXX: Use the 'maxpsum' estimate for precision loss in [KB]?
					if abs(ip) + err > err_stop:
						allSmall = False
						break

				if allSmall:
					# all the roots have been found
					break

			else:
				t += 1

				# define an inner polynomial phi_{r+t+1} = (z-mu) phi_{r+t}
				# phiZeros.append(np.append(phiZeros[-1],mu))

				# define an inner polynomial phi_{r+t+1} = phi_{t+1} phi_{r}
				phiZeros.append(np.append(phiZeros[t],phiZeros[r]))

				# print('inner poly', r+t)
				# print('phiZeros', phiZeros)

		roots = np.array(phiZeros[-1])

		# remove any roots which are not distinct
		removeList = []
		for i, root in enumerate(roots):
			# print(root, roots[i+1:], np.abs(root-roots[i+1:]))
			# print(len(roots[i+1:]) > 0)
			# print(np.any(np.abs(root-roots[i+1:]) < rootTol))
			if len(roots[i+1:]) > 0 and np.any(np.abs(root-roots[i+1:]) < rootTol):
				removeList.append(i)

		roots = np.delete(roots, removeList)

		n = len(roots) # number of distinct roots

		# compute the multiplicities, eq. (1.19) in [KB]
		V = np.column_stack([roots**i for i in range(n)])
		s = [prod(self, f, df, lambda z: z**p, absTol=absTol, relTol=relTol, divMax=divMax)[0] for p in range(n)] # ordinary moments
		multiplicities = np.dot(s, np.linalg.inv(V))
		multiplicities = np.round(multiplicities)

		# remove any roots with multiplicity zero
		zeroArgs = np.where(multiplicities == 0)
		multiplicities = np.delete(multiplicities, zeroArgs)
		roots = np.delete(roots, zeroArgs)

		return tuple(roots), tuple(multiplicities)

	def roots(self, f, df=None, **kwargs):
		return findRoots(self, f, df, **kwargs)

	def demo_roots(self, *args, **kwargs):
		"""
		An animated demonstration of the root finding process using matplotlib.
		Takes all the parameters of :func:`Contour.roots <cxroots.Contours.Contour.roots>` as well as:

		Parameters
		----------
		automaticAnim : bool, optional
			If False (default) then press SPACE to step the animation forward
			If True then the animation will play automatically until all the 
			roots have been found.
		saveFile : str, optional
			If given then the animation will be saved to disk with filename 
			equal to saveFile instead of being shown.
		returnAnim : bool, optional
			If True then the matplotlib animation object will be returned 
			instead of being shown.  Defaults to False.
		"""
		return demo_findRoots(self, *args, **kwargs)

	def show_roots(self, *args, **kwargs):
		roots = self.roots(*args, **kwargs)
		roots.show()

	def print_roots(self, *args, **kwargs):
		roots = self.roots(*args, **kwargs)
		print(roots)

# Reuse docs for roots
try:
	Contour.roots.__doc__ = docstrings.delete_params_s(findRoots.__doc__, ['originalContour'])
except AttributeError:
	# for Python 2.7
	Contour.roots.__func__.__doc__ = docstrings.delete_params_s(findRoots.__doc__, ['originalContour'])



class Circle(Contour):
	"""
	A positively oriented circle in the complex plane.

	Parameters
	----------
	center : complex
		The center of the circle.
	radius : float
		The radius of the circle.
	"""
	def __init__(self, center, radius):
		self.center = center
		self.radius = radius
		self.axisName = ['r']
		self.centralPoint = center

		segments = [ComplexArc(center, radius, 0, 2*pi)]
		super(Circle, self).__init__(segments)

	def __str__(self):
		return 'Circle: center=%.3f, radius=%.3f' % (self.center, self.radius)
	
	def contains(self, z):
		""" Returns True if the point z lies within the contour, False if otherwise """
		return abs(z - self.center) < self.radius

	@property
	def area(self):
		return pi*self.radius**2

	def subdivide(self, axis='r', divisionFactor=0.5):
		"""
		Subdivide the contour

		Parameters
		----------
		axis : str, can only be 'r' (argument kept for consistency with 'subdivisions' method in parent Contour class)
			The axis along which the line subdividing the contour is a constant.
		divisionFactor : float in range (0,1), optional
			Determines the point along 'axis' at which the line dividing the box is placed

		Returns
		-------
		box1 : Annulus
			With inner radius determined by the divisionFactor and outer radius equal to that of the original circle
		box2 : Circle
			With radius equal to the inner radius of box1
		"""
		if axis == 'r' or self.axisName[axis] == 'r':
			box1 = Annulus(self.center, [self.radius*divisionFactor, self.radius])
			box2 = Circle(self.center, self.radius*divisionFactor)
			box1.segments[0] = self.segments[0]
			box1.segments[1]._reversePath = box2.segments[0]
			box2.segments[0]._reversePath = box1.segments[1]

		for box in [box1, box2]:
			box._createdBySubdivisionAxis = axis
			box._parentBox = self
			self._childBoxes = [box1, box2]

		return box1, box2

	def randomPoint(self):
		""" Returns a random point inside the Circle """
		r   = np.random.uniform(0,self.radius)
		phi = np.random.uniform(0,2*pi)
		return r*exp(1j*phi) + self.center

class Annulus(Contour):
	"""
	An annulus in the complex plane with the outer circle positively oriented
	and the inner circle negatively oriented.

	Parameters
	----------
	center : complex
		The center of the annulus in the complex plane.
	radii : list
		A list of length two of the form [inner_radius, outer_radius]
	"""
	def __init__(self, center, radii):
		self.center = center
		self.radii = radii
		self.axisName = ['r', 'phi']

		segments = [ComplexArc(center, radii[1], 0, 2*pi), ComplexArc(center, radii[0], 0, -2*pi)]
		super(Annulus, self).__init__(segments)

	def __str__(self):
		return 'Annulus: center=%.3f, inner radius=%.3f, outer radius=%.3f' % (self.center, self.radii[0], self.radii[1])
	
	@property
	def centralPoint(self):
		# get the central point within the contour
		r = (self.radii[0] + self.radii[1])/2
		return r

	@property
	def area(self):
		return pi*(self.radii[1]**2 - self.radii[0]**2)

	def contains(self, z):
		""" Returns True if the point z lies within the contour, False if otherwise """
		return self.radii[0] < abs(z - self.center) < self.radii[1]

	def subdivide(self, axis, divisionFactor=0.5):
		"""
		Subdivide the contour

		Parameters
		----------
		axis : str, can be either 'r' or 'phi'
			The axis along which the line subdividing the contour is a constant.
		divisionFactor : float in range (0,1), optional
			Determines the point along 'axis' at which the line dividing the box is placed

		Returns
		-------
		boxes : list of contours
			Two annuluses if axis is 'r'.
			Two half-annuluses oriented according to divisionFactor if axis is 'phi'.
		"""
		if axis == 'r' or self.axisName[axis] == 'r':
			midpoint = self.radii[0] + divisionFactor*(self.radii[1]-self.radii[0])
			box1 = Annulus(self.center, [self.radii[0], midpoint])
			box2 = Annulus(self.center, [midpoint, self.radii[1]])

			box1.segments[1] = self.segments[1]
			box2.segments[0] = self.segments[0]
			box1.segments[0]._reversePath = box2.segments[1]
			box2.segments[1]._reversePath = box1.segments[0]

			box1._createdBySubdivisionAxis = axis
			box2._createdBySubdivisionAxis = axis

		elif axis == 'phi' or self.axisName[axis] == 'phi':
			# Subdividing into two radial boxes rather than one to 
			# ensure that an error is raised if one of the new paths
			# is too close to a root
			# XXX: introduce another parameter for phi1

			phi0 = 2*pi*divisionFactor
			phi1 = phi0 + pi

			box1 = AnnulusSector(self.center, self.radii, [phi0, phi1])
			box2 = AnnulusSector(self.center, self.radii, [phi1, phi0])

			box1.segments[0]._reversePath = box2.segments[2]
			box2.segments[2]._reversePath = box1.segments[0]
			box1.segments[2]._reversePath = box2.segments[0]
			box2.segments[0]._reversePath = box1.segments[2]

		for box in [box1, box2]:
			box._createdBySubdivisionAxis = axis
			box._parentBox = self
		self._childBoxes = [box1, box2]

		return box1, box2

	def randomPoint(self):
		""" Returns a random point inside the Annulus """
		r   = np.random.uniform(*self.radii)
		phi = np.random.uniform(0,2*pi)
		return r*exp(1j*phi) + self.center


class AnnulusSector(Contour):
	"""
	A sector of an annulus in the complex plane.
	
	Parameters
	==========
	center : complex
		The center of the annulus sector.
	rRange : list
		List of length two of the form [inner_radius, outer_radius]
	phiRange : list
		List of length two of the form [phi0, phi1].
		The segment of the contour containing inner and outer circular arcs 
		will be joined, counter clockwise from phi0 to phi1.
	"""
	def __init__(self, center, rRange, phiRange):
		self.center = center
		self.axisName = ['r', 'phi']

		if phiRange[0] > phiRange[1]:
			phiRange[1] += 2*pi

		phi0, phi1 = self.phiRange = phiRange

		# r > 0
		r0, r1 = self.rRange = rRange
		if r0 < 0 or r1 <= 0:
			raise ValueError('Radius > 0')

		# verticies [[radius0,phi0],[radius0,phi1],[radius1,phi1],[radius0,phi1]] 
		self.z1 = z1 = center + r0*exp(1j*phi0)
		self.z2 = z2 = center + r1*exp(1j*phi0)
		self.z3 = z3 = center + r1*exp(1j*phi1)
		self.z4 = z4 = center + r0*exp(1j*phi1)

		segments = [ComplexLine(z1,z2),
					ComplexArc(center,r1,phi0,phi1-phi0),
					ComplexLine(z3,z4),
					ComplexArc(center,r0,phi1,phi0-phi1)]

		super(AnnulusSector, self).__init__(segments)

	def __str__(self):
		return 'Polar rectangle: center=%.3f, r0=%.3f, r1=%.3f, phi0=%.3f, phi1=%.3f' % (self.center, self.rRange[0], self.rRange[1], self.phiRange[0], self.phiRange[1])
	
	@property
	def centralPoint(self):
		# get the central point within the contour
		r = (self.rRange[0] + self.rRange[1])/2
		phi = (self.phiRange[0] + self.phiRange[1])/2
		return r*exp(1j*phi)

	@property
	def area(self):
		return (self.rRange[1]**2 - self.rRange[0]**2)*abs(self.phiRange[1] - self.phiRange[0])%(2*pi)/2

	def contains(self, z):
		""" Returns True if the point z lies within the contour, False if otherwise """
		angle = np.angle(z - self.center)%(2*pi) # np.angle maps to [-pi,pi]
		radiusCorrect = self.rRange[0] < abs(z - self.center) < self.rRange[1]
		
		phi = np.mod(self.phiRange, 2*pi)
		if phi[0] > phi[1]:
			angleCorrect = phi[0] < angle <= 2*pi or 0 <= angle < phi[1]
		else:
			angleCorrect = phi[0] < angle < phi[1]

		return radiusCorrect and angleCorrect

	def subdivide(self, axis, divisionFactor=0.5):
		"""
		Subdivide the contour

		Parameters
		----------
		axis : str, can be either 'r' or 'phi'
			The axis along which the line subdividing the contour is a constant.
		divisionFactor : float in range (0,1), optional
			Determines the point along 'axis' at which the line dividing the box is placed

		Returns
		-------
		box1 : AnnulusSector
			If axis is 'r' then phiRange and the inner radius is the same as original AnnulusSector
			with the outer radius determined by the divisionFactor.
			If axis is 'phi' then the rRange and phiRange[0] is the same as the original AnnulusSector
			with phiRange[1] determined by the divisionFactor.
		box2 : AnnulusSector
			If axis is 'r' then phiRange and the outer radius is the same as original AnnulusSector
			with the inner radius determined equal to the outer radius of box1.
			If axis is 'phi' then the rRange and phiRange[1] is the same as the original AnnulusSector
			with phiRange[0] equal to phiRange[1] of box1.
		"""
		r0, r1 = self.rRange
		phi0, phi1 = self.phiRange
		if axis == 0 or axis == self.axisName[0]:
			divisionPoint = r0 + divisionFactor*(r1-r0)
			box1 = AnnulusSector(self.center, [r0, divisionPoint], self.phiRange)
			box2 = AnnulusSector(self.center, [divisionPoint, r1], self.phiRange)

			# reuse line segments from original box where possible
			# this allows the cached integrals to be used
			box1.segments[3] = self.segments[3]
			box2.segments[1] = self.segments[1]
			box1.segments[1]._reversePath = box2.segments[3]
			box2.segments[3]._reversePath = box1.segments[1]

		elif axis == 1 or axis == self.axisName[1]:
			divisionPoint = phi0 + divisionFactor*(phi1-phi0)
			box1 = AnnulusSector(self.center, self.rRange, [phi0, divisionPoint])
			box2 = AnnulusSector(self.center, self.rRange, [divisionPoint, phi1])

			box1.segments[0] = self.segments[0]
			box2.segments[2] = self.segments[2]
			box1.segments[2]._reversePath = box2.segments[0]
			box2.segments[0]._reversePath = box1.segments[2]

		for box in [box1, box2]:
			box._createdBySubdivisionAxis = axis
			box._parentBox = self
		self._childBoxes = [box1, box2]

		return box1, box2

	def randomPoint(self):
		"""Returns a random point inside the contour of the AnnulusSector."""
		r = np.random.uniform(*self.rRange)
		phiRange = np.mod(self.phiRange, 2*pi)
		if phiRange[0] > phiRange[1]:
			phi = random.choice([np.random.uniform(phiRange[0], 2*pi),
								 np.random.uniform(0, phiRange[1])])
		else:
			phi = np.random.uniform(*phiRange)

		return r*exp(1j*phi) + self.center


class Rectangle(Contour):
	"""
	A positively oriented rectangle in the complex plane.
	
	Parameters
	==========
	xRange : list
		List of length 2 giving the range of the rectangle along the real axis.
	yRange : list
		List of length 2 giving the range of the rectangle along the imaginary axis.
	"""
	def __init__(self, xRange, yRange):
		self.xRange = xRange
		self.yRange = yRange
		self.axisName = ['x', 'y']

		self.z1 = z1 = self.xRange[0] + 1j*self.yRange[0]
		self.z2 = z2 = self.xRange[1] + 1j*self.yRange[0]
		self.z3 = z3 = self.xRange[1] + 1j*self.yRange[1]
		self.z4 = z4 = self.xRange[0] + 1j*self.yRange[1]

		segments = [ComplexLine(z1,z2),
					ComplexLine(z2,z3),
					ComplexLine(z3,z4),
					ComplexLine(z4,z1)]
		super(Rectangle, self).__init__(segments)

	def __str__(self):
		return "Rectangle: %.3f+i%.3f, %.3f+i%.3f, %.3f+i%.3f, %.3f+i%.3f"%(self.z1.real, self.z1.imag, self.z2.real, self.z2.imag, self.z3.real, self.z3.imag, self.z4.real, self.z4.imag)

	@property
	def centralPoint(self):
		# get the central point within the contour
		x = (self.xRange[0] + self.xRange[1])/2
		y = (self.yRange[0] + self.yRange[1])/2
		return x + 1j*y

	@property
	def area(self):
		return (self.xRange[1]-self.xRange[0])*(self.yRange[1]-self.yRange[0])

	def contains(self, z):
		""" Returns True if the point z lies within the contour, False if otherwise """
		return self.xRange[0] < z.real < self.xRange[1] and self.yRange[0] < z.imag < self.yRange[1]

	def subdivide(self, axis, divisionFactor=0.5):
		"""
		Subdivide the contour

		Parameters
		----------
		axis : str, can be either 'x' or 'y'
			The axis along which the line subdividing the contour is a constant.
		divisionFactor : float in range (0,1), optional
			Determines the point along 'axis' at which the line dividing the box is placed

		Returns
		-------
		box1 : Rectangle
			If axis is 'x' then box1 has the same yRange and minimum value of xRange as the 
			original Rectangle but the maximum xRange is determined by the divisionFactor.
			If axis is 'y' then box1 has the same xRange and minimum value of yRange as the
			original Rectangle but the maximum yRange is determined by the divisionFactor.
		box2 : Rectangle
			If axis is 'x' then box2 has the same yRange and maximum value of xRange as the 
			original Rectangle but the minimum xRange is equal to the maximum xRange of box1.
			If axis is 'x' then box2 has the same xRange and maximum value of yRange as the 
			original Rectangle but the minimum yRange is equal to the maximum yRange of box1.
		"""
		if axis == 'x' or self.axisName[axis] == 'x':
			midpoint = self.xRange[0] + divisionFactor*(self.xRange[1]-self.xRange[0])
			box1 = Rectangle([self.xRange[0], midpoint], self.yRange)
			box2 = Rectangle([midpoint, self.xRange[1]], self.yRange)

			box1.segments[3] = self.segments[3]
			box2.segments[1] = self.segments[1]
			box1.segments[1]._reversePath = box2.segments[3]
			box2.segments[3]._reversePath = box1.segments[1]

		elif axis == 'y' or self.axisName[axis] == 'y':
			midpoint = self.yRange[0] + divisionFactor*(self.yRange[1]-self.yRange[0])
			box1 = Rectangle(self.xRange, [self.yRange[0], midpoint])
			box2 = Rectangle(self.xRange, [midpoint, self.yRange[1]])

			box1.segments[0] = self.segments[0]
			box2.segments[2] = self.segments[2]
			box1.segments[2]._reversePath = box2.segments[0]
			box2.segments[0]._reversePath = box1.segments[2]

		for box in [box1, box2]:
			box._createdBySubdivisionAxis = axis
			box._parentBox = self
		self._childBoxes = [box1, box2]

		return box1, box2

	def randomPoint(self):
		"""Returns a random point inside the contour of the Rectangle."""
		x = np.random.uniform(*self.xRange)
		y = np.random.uniform(*self.yRange)
		return x + 1j*y


def divisionFactorGen():
	"""A generator for divisionFactors"""
	x = 0.5
	yield x
	for power in [1e1, 1e2, 1e3]:
		for diff in np.linspace(0, 0.5, int(1+power/2))[1:-1]:
			yield x + diff
			yield x - diff



from __future__ import division
import numpy as np
from numpy import inf, pi
import scipy.integrate
import scipy.misc
import warnings

from .CxDerivative import CxDeriv

def prod(C, f, df=None, phi=lambda z:1, psi=lambda z:1, absTol=1e-12, relTol=1e-12, divMax=10, method='quad', callback=None):
	r"""
	Compute the symmetric bilinear form used in (1.12) of [KB]

	.. math::

		<\phi,\psi> = \frac{1}{2i\pi} \oint_C \phi(z) \psi(z) \frac{f'(z)}{f(z)} dz.
	
	References
	----------
	[KB] "Computing the zeros of analytic functions" by Peter Kravanja, Marc Van Barel, Springer 2000
	"""
	N = 1
	k = 0
	I = []
	integrandMax = []

	approx_df = False
	if df is None:
		approx_df = True

	# print('prod:', C)

	if method == 'romb':
		import numdifftools.fornberg as ndf
		# XXX: define err as the difference between successive iterations of the Romberg
		# 	   method for the same number of points?
		while (len(I) < 2 or (abs(I[-2] - I[-1]) > absTol and abs(I[-2] - I[-1]) > relTol*abs(I[-1]))) and k < divMax:
			N = 2*N
			t = np.linspace(0,1,N+1)
			k = int(np.log2(len(t)-1))
			dt = t[1]-t[0]

			# compute/retrieve function evaluations
			fVal = np.array([segment.trapValues(f,k) for segment in C.segments])
			phiVal = np.array([phi(segment(t)) for segment in C.segments])
			psiVal = np.array([psi(segment(t)) for segment in C.segments])

			if approx_df:
				### approximate df/dz with finite difference, see: numdifftools.fornberg
				# interior stencil size = 2*m + 1
				# boundary stencil size = 2*m + 2
				m = 1
				dfdt = [ndf.fd_derivative(fx, t, n=1, m=m) for fx in fVal]
				dfVal = [dfdt[i]/segment.dzdt(t) for i, segment in enumerate(C.segments)]

			else:
				dfVal = np.array([segment.trapValues(df,k) for segment in C.segments])

			segment_integrand = [phiVal[i]*psiVal[i]*dfVal[i]/fVal[i]*segment.dzdt(t) for i, segment in enumerate(C.segments)]
			segment_integral = scipy.integrate.romb(segment_integrand, dx=dt, axis=-1)/(2j*pi)
			I.append(sum(segment_integral))

			# if k>1:
			# 	print(k, 'I', I[-1], 'err', I[-2] - I[-1])

		return I[-1], abs(I[-2] - I[-1])

	elif method == 'quad':
		if approx_df:
			dx = 1e-8
			df = lambda z: scipy.misc.derivative(f, z, dx=dx, n=1, order=3)
			
			# df = CxDeriv(f) # too slow

		I, err = 0, 0
		for segment in C.segments:
			def integrand(t):
				z = segment(t)
				return (phi(z)*psi(z) * df(z)/f(z))/(2j*pi) * segment.dzdt(t)

			# integrate real part
			integrand_real = lambda t: np.real(integrand(t))
			result_real = scipy.integrate.quad(integrand_real, 0, 1, full_output=1, epsabs=absTol, epsrel=relTol)
			I_real, abserr_real, infodict_real = result_real[:3]

			# integrate imaginary part			
			integrand_imag = lambda t: np.imag(integrand(t))
			result_imag = scipy.integrate.quad(integrand_imag, 0, 1, full_output=1, epsabs=absTol, epsrel=relTol)
			I_imag, abserr_imag, infodict_imag = result_imag[:3]

			I   += I_real + 1j*I_imag
			err += abserr_real + 1j*abserr_imag

		return I, abs(err)


class RootError(RuntimeError):
	pass

def count_enclosed_roots(C, f, df=None, NintAbsTol=0.07, integerTol=0.2, divMax=20, method='quad'):
	r"""
	For a function of one complex variable, f(z), which is analytic in and within the contour C,
	return the number of zeros (counting multiplicities) within the contour calculated, using 
	Cauchy's argument principle, as
	
	.. math::

		\frac{1}{2i\pi} \oint_C \frac{f'(z)}{f(z)} dz.

	If df(z), the derivative of f(z), is provided then the above integral is computed directly.
	Otherwise the derivative is approximated using a finite difference approximation implemented
	in Numdifftools <https://pypi.python.org/pypi/Numdifftools>`_.

	The number of points on each segment of the contour C at which f(z) and df(z) are sampled 
	starts at 2+1 and at the k-th iteration the number of points is 2**k+1.  At each iteration 
	the above integral is calculated using `SciPy's implementation of the Romberg method <https://docs.scipy.org/doc/scipy/reference/generated/scipy.integrate.romb.html>`_.
	The routine exits if the difference between successive iterations is < integerTol.

	The number of roots is then the closest integer to the final value of the integral
	and the result is only accepted if the final value of the integral is within integerTol
	of the closest integer.
	
	Parameters
	----------
	C : Contour
		The enclosed_roots function returns the number of roots of f(z) within C
	f : function
		Function of a single variable f(x)
	df : function, optional
		Function of a single variable, df(x), providing the derivative of the function f(x) 
		at the point x.
	integerTol : float, optional
		The evaluation of the Cauchy integral will be accepted if the difference between successive
		iterations is < integerTol and the value of the integral is within integerTol of the 
		closest integer.  Since the Cauchy integral must be an integer it is only necessary to
		distinguish which integer the integral is converging towards.  For this
		reason the integerTol can be set fairly large.
	divMax : int, optional
		The maximum number of divisions before the Romberg integration
		routine of a path exits.

	Returns
	-------
	int
		The number of zeros of f (counting multiplicities) which lie within the contour
	
	References
	----------
	[DL] "A Numerical Method for Locating the Zeros of an Analytic function", 
		L.M.Delves, J.N.Lyness, Mathematics of Computation (1967), Vol.21, Issue 100
	"""
	with warnings.catch_warnings():
		# ignore warnings and catch if I is NaN later
		warnings.simplefilter("ignore")
		I, err = prod(C, f, df, absTol=NintAbsTol, relTol=0, divMax=divMax, method=method)

	if np.isnan(I):
		raise RootError("Result of integral is an invalid value.  Most likely because of a divide by zero error.")

	elif abs(int(round(I.real)) - I.real) < integerTol and abs(I.imag) < integerTol:
		# integral is sufficiently close to an integer
		numberOfZeros = int(round(I.real))
		return numberOfZeros

	else:
		raise RootError("The number of enclosed roots has not converged to an integer")




from __future__ import division
import numpy as np
from numpy import inf, pi
import scipy.integrate
import math

def CxDeriv(f, contour=None):
	"""
	Compute the derivaive of an analytic function using Cauchy's Integral Formula for Derivatives
	"""
	if contour is None:
		from cxroots.Contours import Circle
		C = lambda z0: Circle(z0, 1e-3)
	else:
		C = lambda z0: contour

	def df(z0, n=1):
		integrand = lambda z: f(z)/(z-z0)**(n+1)
		return C(z0).integrate(integrand) * math.factorial(n)/(2j*pi)

	return np.vectorize(df)

def multiplicity_correct(f, df, root, multiplicity, rootErrTol=1e-10):
	"""
	Check a given multplicity by calculating the derivatives
	"""
	if df is None:
		derivs = [f(root)] + [CxDeriv(f)(root,n) for n in range(1,multiplicity+1)]
	else:
		derivs = [f(root), df(root)] + [CxDeriv(f)(root,n) for n in range(2,multiplicity+1)]

	return np.all(np.abs(derivs[:-1]) < rootErrTol) and np.abs(derivs[-1]) > rootErrTol



from __future__ import division

def iterateToRoot(x0, f, df=None, steptol=1e-12, roottol=1e-12, maxIter=20):
	# iterate to a root using initial point x0
	if df is not None:
		try:
			# uses Newton-Raphson method if f and df are given.

			# SciPy implementation
			# import scipy.optimize
			# root = scipy.optimize.newton(f, x0, df, tol=steptol, maxiter=maxIter)
			# err = abs(f(root))
			
			root, err = newton(x0, f, df, steptol, 0, maxIter)

		except (RuntimeError, OverflowError):
			return None
	else:
		# Secant method: 
		x1, x2 = x0, x0 + (x0+1)*1e-8
		root, err = secant(x1, x2, f, steptol, 0, maxIter)

		# XXX: Secant method is very slow to converge.  Use Muller's method instead?
		# Muller's method: uses 3 initial points
		# git+git://github.com/mnishida/PyMuller@master#egg=pymuller
		# from pymuller import Muller
		# muller = Muller(f, 1, args=None, xis=[x0], dxs=[1e-4*(1+x0)], xtol=steptol, ftol=0, maxiter=maxIter)
		# muller()
		# root = muller.roots[0]
		# err = abs(root)

		# import mpmath # XXX: mpmath insists on functions accepting mpc which is inconvenient
		# x1, x2, x3 = x0, x0*(1 + 1e-4) + 1e-4, x0*(1 + 2e-4) + 2e-4
		# root = mpmath.findroot(f, (x1, x2, x3), solver='muller', tol=roottol, verbose=False, verify=False)
		# err = abs(root)

	if err < roottol:
		return root

def newton(x0, f, df, steptol=1e-12, roottol=1e-12, maxIter=20, callback=None):
	"""
	Find an approximation to a point xf such that f(xf)=0 for a 
	scalar function f using Newton-Raphson iteration starting at 
	the point x0.

	Parameters
	----------
	x0 : float or complex
		Initial point for Newton iteration, should be as close as
		possible to a root of f
	f : function
		Function of a single variable f(x)
	df : function
		Function of a single variable, df(x), providing the
		derivative of the function f(x) at the point x
	steptol: float, optional
		Routine will end if the step size, dx, between sucessive
		iterations of x satisfies abs(dx) < steptol
	roottol: float, optional
		The routine will end if abs(f(x)) < roottol
	maxIter : int, optional
		Routine ends after maxIter iterations
	callback : function, optional
		After each iteration the supplied function 
		callback(x, dx, f(x), iteration) will be called where 'x' is the current iteration 
		of the estimated root, 'dx' is the step size between the previous 
		and current 'x' and 'iteration' the number of iterations that have been taken.  
		If the callback function evaluates to True then the routine will end

	Returns
	-------
	xf : float
		The approximation to a root of f
	rooterr : float
		The error of the original function at xf, abs(f(xf))
	"""

	# XXX: Could use deflated polynomials to ensure that known roots are not found again?
	
	# print('--newton--')

	x, y = x0, f(x0)
	for iteration in range(maxIter):
		dx = -y/df(x)
		x += dx
		y  = f(x)

		if callback is not None and callback(x, dx, y, iteration+1):
			break

		# print(iteration, y, abs(y))

		if abs(dx) < steptol or abs(y) < roottol:
			break

	return x, abs(y)

def secant(x1, x2, f, steptol=1e-12, roottol=1e-12, maxIter=30, callback=None):
	"""
	Find an approximation to a point xf such that f(xf)=0 for a 
	scalar function f using the secant method.  The method requires
	two initial points x1 and x2, ideally close to a root
	and proceeds iteratively.

	Parameters
	----------
	x1 : float or complex
		An initial point for iteration, should be close to a 
		root of f.
	x2 : float or complex
		An initial point for iteration, should be close to a 
		root of f.  Should not equal x1.
	f : function
		Function of a single variable f(x)
	steptol: float, optional
		Routine will end if the step size, dx, between sucessive
		iterations of x satisfies abs(dx) < steptol
	roottol: float, optional
		The routine will end if abs(f(x)) < roottol
	maxIter : int, optional
		Routine ends after maxIter iterations
	callback : function, optional
		After each iteration the supplied function 
		callback(x, dx, f(x), iteration) will be called where 'x' is the current iteration 
		of the estimated root, 'dx' is the step size between the previous 
		and current 'x' and 'iteration' the number of iterations that have been taken.  
		If the callback function evaluates to True then the routine will end

	Returns
	-------
	xf : float
		The approximation to a root of f
	rooterr : float
		The error of the original function at xf, abs(f(xf))
	"""

	# As in "Numerical Recipies 3rd Edition" pick the bound with the 
	# smallest function value as the most recent guess
	y1, y2 = f(x1), f(x2)
	if abs(y1) < abs(y2):
		x1, x2 = x2, x1
		y1, y2 = y2, y1

	# print('--secant--')

	for iteration in range(maxIter):
		dx =  -(x2-x1)*y2/(y2-y1)
		x1, x2 = x2, x2 + dx
		y1, y2 = y2, f(x2)

		if callback is not None and callback(x2, dx, y2, iteration+1):
			break

		# print(iteration, x2, abs(dx), abs(y2))

		if abs(dx) < steptol or abs(y2) < roottol:
			break

	return x2, abs(y2)



import unittest
import numpy as np
from scipy import pi

from cxroots import Circle, Rectangle, AnnulusSector
from cxroots.tests.ApproxEqual import roots_approx_equal

class TestRootfindingPolynomial(unittest.TestCase):
	def setUp(self):
		self.roots = roots = [-1.234, 0,  1+1j, 1-1j, 2.345]
		self.multiplicities = [1,1,1,1,1]
		self.f  = lambda z: (z-roots[0])*(z-roots[1])*(z-roots[2])*(z-roots[3])*(z-roots[4])
		self.df = lambda z: (z-roots[1])*(z-roots[2])*(z-roots[3])*(z-roots[4]) + (z-roots[0])*(z-roots[2])*(z-roots[3])*(z-roots[4]) + (z-roots[0])*(z-roots[1])*(z-roots[3])*(z-roots[4]) + (z-roots[0])*(z-roots[1])*(z-roots[2])*(z-roots[4]) + (z-roots[0])*(z-roots[1])*(z-roots[2])*(z-roots[3])
	
		self.Circle = Circle(0,3)
		self.Rectangle = Rectangle([-2,2],[-2,2])
		self.halfAnnulus = AnnulusSector(0, [0.5,3], [-pi/2, pi/2])

	def test_rootfinding_polynomial_circle_fdf(self):
		roots_approx_equal(self.Circle.roots(self.f, self.df), (self.roots, self.multiplicities), decimal=7)

	def test_rootfinding_polynomial_circle_f(self):
		roots_approx_equal(self.Circle.roots(self.f, self.df), (self.roots, self.multiplicities), decimal=7)

	def test_rootfinding_polynomial_rectangle_fdf(self):
		roots_approx_equal(self.Rectangle.roots(self.f, self.df), (self.roots[:-1], self.multiplicities[:-1]), decimal=7)

	def test_rootfinding_polynomial_rectangle_f(self):
		roots_approx_equal(self.Rectangle.roots(self.f, self.df), (self.roots[:-1], self.multiplicities[:-1]), decimal=7)

	def test_rootfinding_polynomial_halfAnnulus_fdf(self):
		roots_approx_equal(self.halfAnnulus.roots(self.f, self.df), (self.roots[2:], self.multiplicities[2:]), decimal=7)

	def test_rootfinding_polynomial_halfAnnulus_f(self):
		roots_approx_equal(self.halfAnnulus.roots(self.f, self.df), (self.roots[2:], self.multiplicities[2:]), decimal=7)

if __name__ == '__main__':
	unittest.main(verbosity=3)



import unittest

from cxroots import AnnulusSector

class TestContains(unittest.TestCase):
	def test_PolarRect_contains(self):
		r0=8.938
		r1=9.625
		phi0=6.126
		phi1=6.519

		z = (9-1.04825594683e-18j)
		C = AnnulusSector(0, [r0,r1], [phi0,phi1])

		# import matplotlib.pyplot as plt
		# plt.scatter((9),(0))
		# C.show()

		self.assertTrue(C.contains(z))

if __name__ == '__main__':
	unittest.main(verbosity=3)



import unittest
import numpy as np
from scipy import cos, sin

from cxroots import Rectangle

class TestCountRoots(unittest.TestCase):
	def setUp(self):
		"""
		Example from "Locating all the Zeros of an Analytic Function in one Complex Variable"
		M.Dellnitz, O.Schutze, Q.Zheng, J. Compu. and App. Math. (2002), Vol.138, Issue 2

		There should be 424 roots inside this contour
		"""
		self.C  = Rectangle([-20.3,20.7], [-20.3,20.7])
		self.f  = lambda z: z**50 + z**12 - 5*sin(20*z)*cos(12*z) - 1
		self.df = lambda z: 50*z**49 + 12*z**11 + 60*sin(12*z)*sin(20*z) - 100*cos(12*z)*cos(20*z)

	def test_countRoots_fdf(self):
		self.assertEqual(self.C.count_roots(self.f, self.df), 424)

	def test_countRoots_f(self):
		self.assertEqual(self.C.count_roots(self.f), 424)

if __name__ == '__main__':
	unittest.main()



"""
A collection of tests

References
----------
[DSZ] "Locating all the Zeros of an Analytic Function in one Complex Variable"
	M.Dellnitz, O.Schutze, Q.Zheng, J. Compu. and App. Math. (2002), Vol.138, Issue 2
[DL] "A Numerical Method for Locating the Zeros of an Analytic function", 
	L.M.Delves, J.N.Lyness, Mathematics of Computation (1967), Vol.21, Issue 100
"""
from __future__ import division
from __future__ import print_function

import numpy as np
from numpy import sin, cos, exp, pi, cosh, sinh, sqrt

np.set_printoptions(linewidth=1e20)

def rootfinding_AnnularCombustionChamber():
	# problem in section 4.2 of [DSZ]
	from cxroots import Rectangle, findRoots

	A = -0.19435
	B = 1000.41
	C = 522463
	T = 0.005

	f = lambda z: z**2 + A*z + B*exp(-T*z) + C
	df = lambda z: 2*z + A - B*T*exp(-T*z)

	rectangle = Rectangle([-15000,5000], [-15000,15000])
	# roots_fdf = rectangle.roots(f, df, integrandUpperBound=np.inf, rootErrTol=1e-8)
	# roots_fdf = rectangle.demo_findRoots(f, df, automaticAnimation=True, integrandUpperBound=np.inf, rootErrTol=1e-8)
	roots_fdf = rectangle.demo_roots(f, df, automaticAnimation=True)
	# roots_f   = rectangle.roots(f)
	# roots_f = rectangle.demo_findRoots(f, automaticAnimation=True, integrandUpperBound=np.inf, rootErrTol=1e-8, M=1)

	# compare with fig 3 of [DSZ]
	import matplotlib.pyplot as plt
	# plt.scatter(np.real(roots_fdf), np.imag(roots_fdf), marker='+')
	plt.scatter(np.real(roots_f), np.imag(roots_f), marker='x')
	plt.show()

def rootfinding_RingOscillator():
	# XXX: Not working, function values too close to zero everywhere?
	# problem in section 4.3 of [DSZ]
	from cxroots import Rectangle, findRoots

	def A(z):
		t = 2.
		return np.array([[-0.0166689-2.12e-14*z, 1/60. + 6e-16*exp(-t*z)*z],
		 		  		 [0.0166659+exp(-t*z)*(-0.000037485+6e-16*z), -0.0166667-6e-16*z]])

	def dA(z):
		t = 2.
		return np.array([[-2.12e-14*z/z, 6e-16*exp(-t*z) - t*6e-16*exp(-t*z)*z],
				   		 [-t*exp(-t*z)*(-0.000037485+6e-16*z)+exp(-t*z)*6e-16, -6e-16*z/z]])
	
	def f(z):
		AVal = np.rollaxis(A(z),-1,0)
		return np.linalg.det(AVal)

	def df(z):
		dAVal = np.rollaxis(dA(z),-1,0)
		return np.linalg.det(dAVal)


	box = Rectangle([-12,0], [-40,40])
	roots_fdf = findRoots(box, f, df, integrandUpperBound=1e6)
	# roots_f = findRoots(box, f)

	print(roots_fdf)

	# # XXX: There don't seem to be any roots within the initial contour?
	# # 	Perhaps there is an issue with the coefficents of z being very small?
	# print(box.count_enclosed_roots(f, df, integerTol=1e-2))
	# print(box.count_enclosed_roots(f, reqEqualZeros=10))

	# compare with fig 4 of [DSZ]
	# import matplotlib.pyplot as plt
	# plt.scatter(np.real(roots_fdf), np.imag(roots_fdf), marker='+')
	# plt.scatter(np.real(roots_f), np.imag(roots_f), marker='x')
	# plt.show()

def simple_test(demo=False):
	from cxroots import Rectangle, showRoots, demo_findRoots, findRoots
	from numpy import pi, sin, cos
	import numpy as np

	rect = Rectangle([-2,2],[-2,2])
	# f  = lambda z: z*(z**10 - 2*z**5 + sin(z)*cos(z/2))8
	# df = lambda z: z*(10*z**9 - 10*z**4 + cos(z)*cos(z/2) - 0.5*sin(z)*sin(z/2)) + z**10 - 2*z**5 + sin(z)*cos(z/2)

	f = lambda z: z**3 * (z-1.2)**2
	df = lambda z: 3*(z)**2 * (z-1.2)**2 + 2*z**3 * (z-1.2)

	roots, multiplicities = findRoots(rect, f, df)
	print('----- Roots -----')
	for root, multiplicity in zip(roots, multiplicities):
		print(multiplicity, root)

	if demo:
		demo_findRoots(rect, f, df, absTol=1e-8, relTol=1e-8)
	# showRoots(rect, f, df)

def test_multiplicity():
	from cxroots.RootFinder import find_multiplicity
	f = lambda z: (z-1)**3*exp(2j*z)
	df = lambda z: 1j*exp(2j*z)*(z-1)**2*(2*z-2-3j)

	print('With f - multiplicity:', find_multiplicity(1, f))
	print('With f & df - multiplicity:', find_multiplicity(1, f, df))


if __name__ == '__main__':
	test_multiplicity()



import numpy as np

def sets_approx_equal(a, b, decimal=7):
	"""
	Test if iterables a and b are approximately the same, up to reordering.

	Works by greedily reordering a and b so that they are the closest to 
	each other element-wise and then using np.testing.assert_almost_equal

	.. np.testing.assert_almost_equal:
	   https://docs.scipy.org/doc/numpy/reference/generated/numpy.testing.assert_almost_equal.html
	"""

	a, b = np.array(a), np.array(b)

	# greedily reorder b so that it is closest to a element-wise
	for ai, a_element in np.ndenumerate(a):
		# find the element of b closest to a_element
		bi = np.argmin(np.abs(b - a_element))
		# put the element of b closest to a_element in the same place as a_element
		b[ai], b[bi] = b[bi], b[ai]

	# use numpy to compare the two arrays element-wise 
	return np.testing.assert_almost_equal(a, b, decimal)


def roots_approx_equal(a, b, decimal = 7):
	a_roots, a_multiplicities = a.roots, a.multiplicities
	b_roots, b_multiplicities = b

	a_roots, b_roots = np.array(a_roots), np.array(b_roots)
	a_multiplicities, b_multiplicities = np.array(a_multiplicities), np.array(b_multiplicities)

	for ai, a_element in np.ndenumerate(a_roots):
		bi = np.argmin(np.abs(b_roots - a_element))
		b_roots[ai], b_roots[bi] = b_roots[bi], b_roots[ai]
		b_multiplicities[ai], b_multiplicities[bi] = b_multiplicities[bi], b_multiplicities[ai]

	np.testing.assert_almost_equal(a_roots, b_roots, decimal)
	np.testing.assert_almost_equal(a_multiplicities, b_multiplicities, decimal)



from __future__ import division
import numpy as np

from .RootFinder import findRootsGen

def demo_findRoots(originalContour, f, df=None, automaticAnim=False, saveFile=None, returnAnim=False, writer=None, **kwargs):
	"""
	An animated demonstration of the root finding process using matplotlib.
	Takes all the parameters of :func:`cxroots.RootFinder.findRoots` as well as:

	Parameters
	----------
	automaticAnim : bool, optional
		If False (default) then press SPACE to step the animation forward
		If True then the animation will play automatically until all the 
		roots have been found.
	saveFile : str, optional
		If given then the animation will be saved to disk with filename 
		equal to saveFile instead of being shown.
	returnAnim : bool, optional
		If True then the matplotlib animation object will be returned 
		instead of being shown.  Defaults to False.
	"""
	import matplotlib.pyplot as plt
	from matplotlib import animation
	fig = plt.gcf()
	ax = plt.gca()

	rootFinder = findRootsGen(originalContour, f, df, **kwargs)
	originalContour.plot(linecolor='k', linestyle='--')
	originalContour.sizePlot()

	def update_frame(args):
		roots, multiplicities, boxes, numberOfRemainingRoots = args
		# print(args)

		plt.cla() # clear axis
		originalContour.plot(linecolor='k', linestyle='--')
		originalContour.sizePlot()
		for box, numberOfEnclosedRoots in boxes:
			if not hasattr(box, '_color'):
				cmap = plt.get_cmap('jet')
				box._color = cmap(np.random.random())
			
			plt.text(box.centralPoint.real, box.centralPoint.imag, numberOfEnclosedRoots)
			box.plot(linecolor=box._color)

		plt.scatter(np.real(roots), np.imag(roots))

		rootsLabel = ax.text(0.02, 0.95, 'Zeros remaining: %i'%numberOfRemainingRoots, transform=ax.transAxes)

		fig.canvas.draw()

	if saveFile:
		automaticAnim = True

	if automaticAnim or returnAnim:
		anim = animation.FuncAnimation(fig, update_frame, frames=rootFinder, interval=500)

		if returnAnim:
			return anim

	else:
		def draw_next(event):
			if event.key == ' ':
				update_frame(next(rootFinder)) 
		fig.canvas.mpl_connect('key_press_event', draw_next)

	if saveFile:
		anim.save(filename=saveFile, fps=1, dpi=200, writer=writer)
		plt.close()
	else:
		plt.show()



import unittest

from cxroots.CxDerivative import multiplicity_correct

class TestMultiplicity(unittest.TestCase):
	def setUp(self):
		self.f = lambda z: (z-2)**2*(z-1)**5 
		self.df = lambda z: 2*(z-2)*(z-1)**5 + 5*(z-2)**2*(z-1)**4

	def test_multiplicity_check1_f(self):
		self.assertFalse(multiplicity_correct(self.f, None, 2, 1))

	def test_multiplicity_check2_f(self):
		self.assertTrue(multiplicity_correct(self.f, None, 2, 2))

	def test_multiplicity_check3_f(self):
		self.assertFalse(multiplicity_correct(self.f, None, 2, 3))

	def test_multiplicity_check4_f(self):
		self.assertFalse(multiplicity_correct(self.f, None, 1, 4))

	def test_multiplicity_check5_f(self):
		self.assertTrue(multiplicity_correct(self.f, None, 1, 5))

	def test_multiplicity_check6_f(self):
		self.assertFalse(multiplicity_correct(self.f, None, 1, 6))

	def test_multiplicity_check1_df(self):
		self.assertFalse(multiplicity_correct(self.f, self.df, 2, 1))

	def test_multiplicity_check2_df(self):
		self.assertTrue(multiplicity_correct(self.f, self.df, 2, 2))

	def test_multiplicity_check3_df(self):
		self.assertFalse(multiplicity_correct(self.f, self.df, 2, 3))

	def test_multiplicity_check4_df(self):
		self.assertFalse(multiplicity_correct(self.f, self.df, 1, 4))

	def test_multiplicity_check5_df(self):
		self.assertTrue(multiplicity_correct(self.f, self.df, 1, 5))

	def test_multiplicity_check6_df(self):
		self.assertFalse(multiplicity_correct(self.f, self.df, 1, 6))

if __name__ == '__main__':
	unittest.main()



from __future__ import division
import numpy as np
import cmath
from scipy import pi, exp, sin
import scipy
from numpy.random import uniform
import itertools
from collections import deque
import warnings

from .IterativeMethods import iterateToRoot
from .CountRoots import prod, RootError
from .RootResult import RootResult
from .CxDerivative import multiplicity_correct

from .Misc import doc_tab_to_space, docstrings

@docstrings.get_sectionsf('findRootsGen')
@docstrings.dedent
@doc_tab_to_space
def findRootsGen(originalContour, f, df=None, guessRoots=[], guessRootSymmetry=None, 
	newtonStepTol=1e-14, newtonMaxIter=50, rootErrTol=1e-10, absTol=0, relTol=1e-12, 
	divMax=20, integerTol=0.07, NintAbsTol=0.07, M=5, intMethod='quad'):
	"""
	A generator which at each step takes a contour and either finds 
	all the zeros of f within it or subdivides it further.  Based
	on the algorithm in [KB]

	Parameters
	----------
	originalContour : Contour
		The contour C which bounds the region in which all the 
		roots of f(z) are sought.
	f : function
		A function of a single complex variable, z, which is 
		analytic within C and has no poles or roots on C.
	df : function, optional
		A function of a single complex variable which is the 
		derivative of the function f(z). If df is not given 
		then it will be approximated with a finite difference 
		formula.
	guessRoots : list, optional
		A list of known roots or, if the multiplicity is known, 
		a list of (root, multiplicity) tuples.
	guessRootSymmetry : function, optional
		A function of a single complex variable, z, which 
		returns a list of all points which are expected to be 
		roots of f, given that z is a root of f.
	newtonStepTol : float, optional
		The required accuracy of the root.
		The iterative method used to give a final value for each
		root will exit if the step size, dx, between sucessive 
		iterations satisfies abs(dx) < newtonStepTol
	newtonMaxIter : int, optional
		The iterative method used to give a final value for each
		root will exit if the number of iterations exceeds newtonMaxIter.
	rootErrTol : float, optional
		A complex value z is considered a root if abs(f(z)) < rootErrTol
	absTol : float, optional
		Absolute error tolerance used by the contour integration.
	relTol : float, optional
		Relative error tolerance used by the contour integration.
	divMax : int, optional
		If the Romberg integration method is used then divMax is the
		maximum number of divisions before the Romberg integration
		routine of a path exits.
	integerTol : float, optional
		A number is considered an integer if it is within 
		integerTol of an integer.  Used when calculating the 
		number of roots within a contour.
	NintAbsTol : float, optional
		The absolute error tolerance used for the contour 
		integration when determining the number of roots 
		within a contour.  Since the result of this integration
		must be an integer it can be much less accurate
		than usual.
	M : int, optional
		If the number of roots (including multiplicites) within a
		contour is greater than M then the contour is subdivided
		further.  M must be greater than or equal to the largest 
		multiplcity of any root.  
	intMethod : str, optional
		Either 'quad' to integrate using scipy.integrate.quad or
		'romb' to integrate using Romberg's method.

	Yields
	------
	list
		Roots of f(z) within the contour C
	list
		Multiplicites of roots
	deque
		The contours which still contain roots
	int
		Remaining number of roots to be found within the contour

	References
	----------
	[KB] Peter Kravanja, Marc Van Barel, "Computing the Zeros of 
	Anayltic Functions", Springer (2000)
	"""
	try:
		# total number of zeros, including multiplicities
		totNumberOfRoots = originalContour.count_roots(f, df, NintAbsTol, integerTol, divMax)
		originalContour._numberOfRoots = totNumberOfRoots
	except RuntimeError:
		raise RuntimeError("""
			Integration along the initial contour has failed.  There is likely a root on or close to the initial contour
			Try changing the initial contour, if possible.""")

	smallBoxWarning = False
	roots = []
	multiplicities = []
	failedBoxes = []
	boxes = deque()
	boxes.append((originalContour,totNumberOfRoots))

	def subdivide(parentBox):
		"""
		Given a contour, parentBox, subdivide it into multiple contours.
		"""
		numberOfRoots = None
		for subBoxes in parentBox.subdivisions():
			# if a box has already been used and caused an error then skip it
			failedBoxDesc = list(map(str, failedBoxes))
			if np.any([boxDesc in failedBoxDesc for boxDesc in list(map(str, subBoxes))]):
				continue

			# XXX: if a root is near to a box then subdivide again?

			try:
				numberOfRoots = [box.count_roots(f, df, NintAbsTol, integerTol, divMax) for box in np.array(subBoxes)]
				if parentBox._numberOfRoots == sum(numberOfRoots):
					break
			except RootError:
				# If the number of zeros within either of the new contours is not an integer then it is
				# likely that the introduced line which subdivides 'parentBox' lies on a zero.
				# To avoid this we will try to place the subdividing line at a different point along 
				# the division axis
				continue

		if numberOfRoots is None or parentBox._numberOfRoots != sum(numberOfRoots):
			# The list of subdivisions has been exhaused and still the number of enclosed zeros does not add up 
			raise RuntimeError("""Unable to subdivide box:
				\t%s
				""" % parentBox)

		boxes.extend([(box, numberOfRoots[i]) for i, box in enumerate(subBoxes) if numberOfRoots[i] != 0])

		for i, box in enumerate(subBoxes):
			box._numberOfRoots = numberOfRoots[i]


	def addRoot(root, multiplicity=None, useGuessRootSymmetry=None):
		# check that the root we have found is distinct from the ones we already have
		if not roots or np.all(abs(np.array(roots) - root) > newtonStepTol):
			# add the root to the list if it is within the original box
			if originalContour.contains(root):
				roots.append(root)
				if multiplicity is None:
					from .Contours import Circle
					C = Circle(root, 1e-3)
					# multiplicity, = C.approximate_roots(f, df, absTol, relTol, integerTol, divMax, rootTol=newtonStepTol)[1]
					multiplicity, = C.approximate_roots(f, df, rootTol=newtonStepTol)[1]

				multiplicities.append(multiplicity.real)

			# check to see if there are any other roots implied by the given symmetry
			if guessRootSymmetry is not None:
				for x0 in guessRootSymmetry(root):
					root = iterateToRoot(x0, f, df, newtonStepTol, rootErrTol, newtonMaxIter)
					if root is not None:
						addRoot(root)


	# Add given roots 
	for guess in guessRoots:
		if hasattr(guess, '__iter__'):
			root, multiplicity = guess
			if not multiplicity_correct(f, df, root, multiplicity):
				continue

		else:
			root, multiplicity = guess, None

		# XXX: refine root with Newton-Raphson?
		
		if abs(f(root)) < rootErrTol:
			addRoot(root, multiplicity)

	# yield so that the animation shows the first frame
	totFoundRoots = sum(int(round(multiplicity.real)) for root, multiplicity in zip(roots, multiplicities))
	yield roots, multiplicities, boxes, totNumberOfRoots - totFoundRoots

	# print('Tot number of Roots', totNumberOfRoots)

	while boxes:
		box, numberOfRoots = boxes.pop()

		def remove_relations(C):
			# remove itself
			try:
				boxes.remove((C, C._numberOfRoots))
			except ValueError:
				pass

			# get all direct relations
			# siblings:
			relations = C._parentBox._childBoxes 
			relations.remove(C)

			# children:
			if hasattr(C, '_childBoxes'):
				relations.extend(C._childBoxes)

			# interate over all relations
			for relation in relations:
				remove_relations(relation)

		# if a known root is too near this box then reverse the subdivision that created it 
		t = np.linspace(0,1,10001) 
		if np.any([np.any(np.abs(box(t) - root) < 1e-3) for root in roots]):
			# remove the box and any relations
			remove_relations(box)

			# put the parent box back into the list of boxes to subdivide again
			parent = box._parentBox
			boxes.append((parent, parent._numberOfRoots))

			# compute the root multiplicity directly
			approxRootMultiplicity = None

			# do not use this box again
			failedBoxes.append(box)

			continue

		# print(numberOfRoots, box)

		# if box is smaller than the newtonStepTol then just assume that the root is
		# at the center of the box, print a warning and move on
		if box.area < newtonStepTol:
			smallBoxWarning = True
			if smallBoxWarning is False:
				warnings.warn('The area of the interior of a contour containing %i is smaller than newtonStepTol!  \
					\nThe center of the box has been recored as a root of multiplicity %i but this could not be verified.  \
					\nThe same assumption will be made for future contours this small without an additional warning.  \
					\nrootErrTol may be too small.'%(numberOfRoots, numberOfRoots))
			root = box.centralPoint
			addRoot(root,  multiplicity=numberOfRoots)
			continue

		# if all the roots within the box have been located then coninue to the next box
		numberOfKnownRootsInBox = sum([int(round(multiplicity.real)) for root, multiplicity in zip(roots, multiplicities) if box.contains(root)])
		# print('box', box, 'N', numberOfRoots, 'knownN', numberOfKnownRootsInBox)
		
		if numberOfRoots == numberOfKnownRootsInBox:
			continue

		# if there are too many roots within the contour then subdivide
		# if there are any known roots within the contour then subdivide (so as not to waste resources re-approximating them)
		if numberOfKnownRootsInBox > 0 or numberOfRoots > M:
			subdivide(box)

		else:
			# approximate the roots in this box
			approxRoots, approxRootMultiplicities = box.approximate_roots(f, df, absTol, relTol, integerTol, divMax, rootTol=newtonStepTol)
			for approxRoot, approxRootMultiplicity in list(zip(approxRoots, approxRootMultiplicities)):
				# XXX: if the approximate root is not in this box then desregard and redo the subdivision?

				# attempt to refine the root
				root = iterateToRoot(approxRoot, f, df, newtonStepTol, rootErrTol, newtonMaxIter)

				# print('approx', approxRoot, 'refined root', root)

				if abs(f(approxRoot)) < rootErrTol and (root is None or abs(f(approxRoot)) < abs(f(root))):
					# stick with the original approximation
					root = approxRoot

				if root is not None:
					# if the root is very close to the contour then disregard this contour and compute the root multiplicity directly
					# XXX: implement a distance function returning the shortest distance from a point to any point on a contour
					t = np.linspace(0,1,10001) 
					if np.any(np.abs(box(t) - root) < 1e-3):
						# remove the box and any relations
						remove_relations(box)

						# put the parent box back into the list of boxes to subdivide again
						parent = box._parentBox
						boxes.append((parent, parent._numberOfRoots))

						# compute the root multiplicity directly
						approxRootMultiplicity = None

						# do not use this box again
						failedBoxes.append(box)
					
					# if we found a root add it to the list of known roots
					addRoot(root, approxRootMultiplicity)

			# if we haven't found all the roots then subdivide further
			numberOfKnownRootsInBox = sum([int(round(multiplicity.real)) for root, multiplicity in zip(roots, multiplicities) if box.contains(root)])
			if numberOfRoots != numberOfKnownRootsInBox and box not in failedBoxes:
				subdivide(box)

		totFoundRoots = sum(int(round(multiplicity.real)) for root, multiplicity in zip(roots, multiplicities))
		yield roots, multiplicities, boxes, totNumberOfRoots - totFoundRoots

	# yield one more time so that the animation shows the final frame
	yield roots, multiplicities, boxes, totNumberOfRoots - totFoundRoots

	if totNumberOfRoots == 0:
		yield [], [], deque(), 0

@docstrings.dedent
@doc_tab_to_space
def findRoots(originalContour, f, df=None, **kwargs):
	"""
	Find all the roots of the complex analytic function f within the given contour.

	Parameters
	----------
	%(findRootsGen.parameters)s

	Returns
	-------
	result : rootResult
		A container for the roots and their multiplicities
	"""
	rootFinder = findRootsGen(originalContour, f, df, **kwargs)
	for roots, multiplicities, boxes, numberOfRemainingRoots in rootFinder:
		pass
	return RootResult(roots, multiplicities, originalContour)




from __future__ import division
import numpy as np

class RootResult(object):
	"""
	A container to hold roots and multiplicities.

	Attributes
	----------
	roots : list
		List of roots
	multiplicities : list
		List of multiplicities where the ith element of the list
		is the multiplicity of the ith element of roots.
	originalContour : Contour
		The contour bounding the region in which the roots
		were found.
	"""
	def __init__(self, roots, multiplicities, originalContour):
		self.roots = roots
		self.multiplicities = multiplicities
		self.originalContour = originalContour

	def show(self, saveFile=None):
		"""
		Plot the roots on the complex plane.

		Parameters
		----------
		saveFile : str, optional
			If provided the plot of the roots will be saved with
			file name saveFile instead of being shown.

		Example
		-------
		.. plot::
			:include-source:
			
			from cxroots import Circle
			C = Circle(0, 2)
			f = lambda z: z**6 + z**3
			r = C.roots(f)
			r.show()
		"""
		import matplotlib.pyplot as plt
		self.originalContour.plot(linecolor='k', linestyle='--')
		plt.scatter(np.real(self.roots), np.imag(self.roots), color='k', marker='x')

		if saveFile is not None:
			plt.savefig(saveFile)
		else:
			plt.show()

	def __str__(self):
		roots, multiplicities = np.array(self.roots), np.array(self.multiplicities)

		# reorder roots
		sortargs = np.argsort(roots)
		roots, multiplicities = roots[sortargs], multiplicities[sortargs]

		s =  ' Multiplicity |               Root              '
		s+='\n------------------------------------------------'

		for i, root in np.ndenumerate(roots):
			if root.real < 0:
				s += '\n{: ^14d}| {:.12f} {:+.12f}i'.format(int(multiplicities[i]), root.real, root.imag)
			else:
				s += '\n{: ^14d}|  {:.12f} {:+.12f}i'.format(int(multiplicities[i]), root.real, root.imag)

		return s



import unittest
from numpy import exp, sin, cos

from cxroots import Circle
from cxroots.CxDerivative import multiplicity_correct
from cxroots.tests.ApproxEqual import roots_approx_equal

class TestGuessRoots(unittest.TestCase):
	def setUp(self):
		self.C = Circle(0, 3)
		self.f = lambda z: (z-2.5)**2 * (exp(-z)*sin(z/2)-1.2*cos(z))

		self.roots = [2.5,
					  1.44025113016670301345110737, 
					  -0.974651035111059787741822566 - 1.381047768247156339633038236j,
					  -0.974651035111059787741822566 + 1.381047768247156339633038236j]
		self.multiplicities = [2,1,1,1]

	def test_guessRoots_1(self):
		# root and multiplicity given
		roots_approx_equal(self.C.roots(self.f, guessRoots=[(2.5,2)]), (self.roots, self.multiplicities), decimal=12)

	def test_guessRoots_2(self):
		# just root
		roots_approx_equal(self.C.roots(self.f, guessRoots=[2.5]), (self.roots, self.multiplicities), decimal=12)

	def test_guessRoots_3(self):
		# wrong roots
		roots_approx_equal(self.C.roots(self.f, guessRoots=[3,1]), (self.roots, self.multiplicities), decimal=12)

	def test_guessRoots_4(self):
		# wrong multiplicity
		roots_approx_equal(self.C.roots(self.f, guessRoots=[(2.5,1)]), (self.roots, self.multiplicities), decimal=12)

	def test_guessRoots_5(self):
		# wrong multiplicity
		roots_approx_equal(self.C.roots(self.f, guessRoots=[(2.5,3)]), (self.roots, self.multiplicities), decimal=12)


if __name__ == '__main__':
	unittest.main()



#!/usr/bin/env python
import os
import sys
import shutil
import unittest
from distutils.core import setup, Command
from distutils.extension import Extension
from numpy.distutils.misc_util import get_numpy_include_dirs

packages = ['cxroots', 'cxroots.tests']

# get the version
exec(open('cxroots/version.py').read())

setup(
    name = 'cxroots',
    version = __version__,
    description = 'Find all the roots of a function within a contour in the complex plane',
    author = 'Robert Parini',
    url = 'https://github.com/rparini/cxroots',
    license = 'BSD',
    packages = packages,
    platforms = ['all'],
    install_requires = ['numpy', 'scipy', 'docrep'],
    classifiers=[
	    'Development Status :: 4 - Beta',
	    'Intended Audience :: Science/Research',
	    'Topic :: Scientific/Engineering :: Mathematics',
	    'License :: OSI Approved :: BSD License',
	    'Programming Language :: Python :: 2.7',
	    'Programming Language :: Python :: 3',
	],
	keywords='roots zeros complex analytic functions',
	
)



import unittest
import numpy as np

from cxroots.IterativeMethods import newton
from scipy import pi, cos, sin

class TestNewton(unittest.TestCase):
	def test_newton(self):
		# result from keisan online calculator: http://keisan.casio.com/exec/system/1244946907
		f  = lambda x: cos(x)-x
		df = lambda x: -sin(x)-1

		iterations = []
		callback = lambda x, dx, y, iteration: iterations.append(x)
		x, err = newton(pi/4, f, df, callback=callback)
		iterations.append(x)
		
		correct_iterations = [0.73953613351523830094,
							  0.7390851781060101829533,
				  			  0.7390851332151610866198,
				  			  0.7390851332151606416553]

		np.testing.assert_almost_equal(iterations, correct_iterations, decimal=15)

if __name__ == '__main__':
	unittest.main()



import unittest
from numpy import exp, sin, cos

from cxroots import Circle
from cxroots.CxDerivative import multiplicity_correct
from cxroots.tests.ApproxEqual import roots_approx_equal

class TestGuessRootSymmetry(unittest.TestCase):
	def setUp(self):
		self.C = Circle(0, 3)
		self.f = lambda z: z**4 + z**3 + z**2 + z

		self.roots = [0,-1,1j,-1j]
		self.multiplicities = [1,1,1,1]

	def test_guessRootSymmetry_1(self):
		symmetry = lambda z: [z.conjugate()]
		roots_approx_equal(self.C.roots(self.f, guessRootSymmetry=symmetry), (self.roots, self.multiplicities), decimal=12)

	def test_guessRootSymmetry_2(self):
		# wrong symmetry
		symmetry = lambda z: [z+1]
		roots_approx_equal(self.C.roots(self.f, guessRootSymmetry=symmetry), (self.roots, self.multiplicities), decimal=12)


if __name__ == '__main__':
	unittest.main()



from __future__ import division
import random
import warnings
import numpy as np
import scipy.integrate
import scipy.linalg
from scipy import pi, exp, sin, log
import scipy

class ComplexPath(object):
	""" A base class for paths in the complex plane """
	def __init__(self):
		self._integralCache = {}
		self._contArgCache = {}

		self._trapValuesCache = {}

	def trapValues(self, f, k):
		"""
		2**k+1 is the number of required points for the function f to
		be evaluated at.
		"""

		if f in self._trapValuesCache.keys():
			vals = self._trapValuesCache[f]
			vals_k = int(np.log2(len(vals)-1))
			
			if vals_k == k:
				return vals
			elif vals_k > k:
				return vals[::2**(vals_k-k)]
			else:
				t = np.linspace(0, 1, 2**k+1)
				vals = np.empty(2**k+1, dtype=np.complex128)
				vals.fill(np.nan)
				vals[::2**(k-vals_k)] = self._trapValuesCache[f]
				vals[np.isnan(vals)] = f(self(t[np.isnan(vals)]))

				# cache values
				self._trapValuesCache[f] = vals
				return vals

		else:
			t = np.linspace(0, 1, 2**k+1)
			vals = f(self(t))
			self._trapValuesCache[f] = vals
			return vals


	def plot(self, N=100, linecolor='b', linestyle='-'):
		"""
		Use matplotlib to plot, but not show, the path as a 
		2D plot in the Complex plane.  To show use pyplot.show()

		Parameters
		----------
		N : int, optional
			The number of points to use when plotting the path.  Default is 100
		linecolor : optional
			The colour of the plotted path, passed to the pyplot.plot function 
			as the keyword argument of 'color'.
		linestyle : str, optional
			The line style of the plotted path, passed to the pyplot.plot 
			function as the keyword argument of 'linestyle'.  Default is '-' 
			which corresponds to a solid line.
		"""
		import matplotlib.pyplot as plt
		t = np.linspace(0,1,N)
		path = self(t)
		plt.plot(path.real, path.imag, color=linecolor, linestyle=linestyle)
		plt.xlabel('Re[$z$]', size=16)
		plt.ylabel('Im[$z$]', size=16)
		plt.gca().set_aspect(1)
		plt.tight_layout()

		# add arrow to indicate direction of path
		arrow_direction = (self(0.51) - self(0.5))/abs(self(0.51) - self(0.5))
		arrow_extent = 1e-6*arrow_direction
		ymin, ymax = plt.gca().get_ylim()
		xmin, xmax = plt.gca().get_xlim()
		head_length = max(abs(ymax - ymin), abs(xmax - xmin))/40.
		plt.arrow(self(0.5).real, self(0.5).imag,
				  arrow_extent.real, arrow_extent.imag,
				  head_width=head_length*2/3., head_length=head_length, 
				  fc=linecolor, ec=linecolor)

	def show(self, *args, **kwargs):
		""" Shows the path as a 2D plot in the complex plane using 
		the same arguments as the plot method """
		import matplotlib.pyplot as plt
		self.plot(*args, **kwargs)
		plt.show()

	def integrate(self, f, absTol=0, relTol=1e-12, rombergDivMax=10, method='quad', show=False):
		"""
		Integrate the function f along the path using SciPy's Romberg
		algorithm.  The value of the integral is cached and will be
		reused if the method is called with same function f and tol. 

		Parameters
		----------
		f : function of a single complex variable
		tol : float, optional
			The absolute tolerance passed to SciPy's Romberg function.
			Default is 1e-8.
		rombergDivMax : int, optional
			The maximum order of extrapolation passed to SciPy's Romberg function

		Returns
		-------
		integral : complex
			The integral of the function f along the path.
		"""

		args = (f, absTol, relTol, rombergDivMax, method)
		if args in self._integralCache.keys():
			integral = self._integralCache[args]

		elif hasattr(self, '_reversePath') and args in self._reversePath._integralCache:
			# if we have already computed the reverse of this path
			integral = -self._reversePath._integralCache[args]

		else:			
			# suppress accuracy warnings
			with warnings.catch_warnings():
				warnings.simplefilter("ignore")
				integrand = lambda t: f(self(t))*self.dzdt(t)

				if method == 'romb':
					integral = scipy.integrate.romberg(integrand, 0, 1, tol=absTol, rtol=relTol, divmax=rombergDivMax, show=show)
				elif method == 'quad':
					integrand_real = lambda t: np.real(integrand(t))
					integrand_imag = lambda t: np.imag(integrand(t))

					integral_real, abserr_real = scipy.integrate.quad(integrand_real, 0, 1, epsabs=absTol, epsrel=relTol)
					integral_imag, abserr_imag = scipy.integrate.quad(integrand_imag, 0, 1, epsabs=absTol, epsrel=relTol)
					integral = integral_real + 1j*integral_imag

			if np.isnan(integral):
				raise RuntimeError('The integral along the segment %s is NaN.\
					\nThis is most likely due to a root being on or very close to the path of integration.'%self)

			self._integralCache[args] = integral

		return integral


class ComplexLine(ComplexPath):
	""" A straight line in the complex plane from a to b """
	def __init__(self, a, b):
		self.a, self.b = a, b
		self.dzdt = lambda t: self.b-self.a
		super(ComplexLine, self).__init__()

	def __str__(self):
		return 'ComplexLine from %.3f+%.3fi to %.3f+%.3fi' % (self.a.real, self.a.imag, self.b.real, self.b.imag)

	def __call__(self, t):
		""" The parameterization of the line in the variable t, where 0 <= t <= 1 """
		return self.a + t*(self.b-self.a)

class ComplexArc(ComplexPath):
	""" An arc with center z0, radius R, initial angle t0 and change of angle dt """
	def __init__(self, z0, R, t0, dt):
		self.z0, self.R, self.t0, self.dt = z0, R, t0, dt
		self.dzdt = lambda t: 1j*self.dt*self.R*exp(1j*(self.t0 + t*self.dt))
		super(ComplexArc, self).__init__()

	def __str__(self):
		return 'ComplexArc: z0=%.3f, R=%.3f, t0=%.3f, dt=%.3f' % (self.z0, self.R, self.t0, self.dt)

	def __call__(self, t):
		""" The parameterization of the arc in the variable t, where 0 <= t <= 1 """
		return self.R*exp(1j*(self.t0 + t*self.dt)) + self.z0



import unittest
import numpy as np
from scipy import cos, sin

from cxroots import Circle, Rectangle
from cxroots import CxDeriv

class TestDerivative1(unittest.TestCase):
	def setUp(self):
		self.f  = lambda z: z**10 - 2*z**5 + sin(z)*cos(z/2)
		self.df = lambda z: 10*(z**9 - z**4) + cos(z)*cos(z/2) - 0.5*sin(z)*sin(z/2)

	def test_circle(self):
		C = Circle(0, 2)
		z = np.array([-1.234, 0.3+1j, 0.1j, -0.9-0.5j])

		df_approx = CxDeriv(self.f, C)
		np.testing.assert_almost_equal(df_approx(z, 1), self.df(z), decimal=1e-12)

	def test_rect(self):
		C = Rectangle([-1.5,1.5],[-2,2])
		z = np.array([-1.234, 0.3+1j, 0.1j, -0.9-0.5j])

		df_approx = CxDeriv(self.f, C)
		np.testing.assert_almost_equal(df_approx(z, 1), self.df(z), decimal=1e-12)

	def test_smallCircle(self):
		z = 0.3+1j

		df_approx = CxDeriv(self.f)
		np.testing.assert_almost_equal(df_approx(z, 1), self.df(z), decimal=1e-12)

if __name__ == '__main__':
	unittest.main()
